@@ -3,7 +3,7 @@
  * you can get it at "http://www.nongnu.org/clanbomber".
  *
  * Copyright (C) 2003, 2004, 2007 mass
- * Copyright (C) 2009 Rene Lopez <rsl@members.fsf.org>
+ * Copyright (C) 2009, 2010 Rene Lopez <rsl@members.fsf.org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -31,6 +31,7 @@
 #include "MapTile.h"
 
 #include <algorithm>
+#include <assert.h>
 
 #define random(num) (rand() % num)
 
@@ -68,6 +69,7 @@ enum {
 
 AIPathItem::AIPathItem(const Controller_AI_mass* cai, CL_Vector* v, CL_Vector f, AIAction act)
 {
+  assert(v != NULL);
   map_v = v;
   ai = cai;
   was_passing = false;
@@ -98,6 +100,7 @@ AIPathItem::AIPathItem(const Controller_AI_mass* cai, CL_Vector* v, CL_Vector f,
 
 AIPathItem::~AIPathItem()
 {
+  assert(map_v != NULL);
   delete map_v;
 }
 
@@ -278,6 +281,7 @@ AIPath::AIPath(Controller_AI_mass* cai, const std::vector<CL_Vector*> &way, AIAc
 
   for (int i = 0; i < way.size(); i++) {
     if (i == 0) {
+      assert(i+1 < way.size());
       path.push_back(new AIPathItem(cai, way[i], *way[i+1], first_act));
     }
     else {
@@ -285,6 +289,7 @@ AIPath::AIPath(Controller_AI_mass* cai, const std::vector<CL_Vector*> &way, AIAc
         path.push_back (new AIPathItem (cai, way[i], *(way[i]), last_act));
       }
       else {
+        assert(i+1 < way.size());
         path.push_back (new AIPathItem (cai, way[i], *(way[i+1]), step_act));
       }
     }
@@ -306,6 +311,10 @@ AIPath::~AIPath()
       delete path;
       path = NULL;
       }*/
+  for(std::vector<AIPathItem*>::iterator iter = path.begin();
+      iter != path.end(); iter++) {
+    delete *iter;
+  }
 }
 
 bool AIPath::passed() const
@@ -315,6 +324,7 @@ bool AIPath::passed() const
       return true;
       }*/
 
+  assert(curr_item >= 0);
   return ((curr_item + 1) >= path.size());
 }
 
@@ -830,13 +840,12 @@ void Controller_AI_mass::assign_path()
   }
   else {
     //printf ("sure position.\n");
-    //CL_List<CL_Vector>* sure_path = sure_position();
     std::vector<CL_Vector*>* sure_path = sure_position();
     if (sure_path != NULL) {
       //printf ("sure pos ok.\n");
       curr_path = new AIPath(this, *sure_path, AI_panic);
       sure_path->clear();
-      //delete sure_path;
+      delete sure_path;
     }
     return;
   }
@@ -857,15 +866,10 @@ void Controller_AI_mass::assign_path()
   }
 
   //printf ("placing bomb.\n");
-  //CL_List<CL_Vector>* bombpath = bomb_path();
   std::vector<CL_Vector*>* bombpath = bomb_path();
   if (bombpath != NULL) {
     //printf ("bomb path found, path len (%d).\n", bombpath->get_num_items());
     curr_path = new AIPath (this, *bombpath, AI_put);
-    /*for(std::vector<CL_Vector*>::iterator iter = bombpath->begin();
-        iter != bombpath->end(); iter++) {
-      delete *iter;
-      }*/
     bombpath->clear();
     delete bombpath;
     return;
@@ -894,32 +898,25 @@ static int vlen(CL_Vector v1, CL_Vector v2)
 
 std::vector<CL_Vector*>* Controller_AI_mass::seek_util()
 {
-  std::vector<CL_Vector*>* ret_path;
+  std::vector<CL_Vector*>* ret_path = NULL;
   CL_Vector b_v(bomber->get_map_x(), bomber->get_map_y());
   std::list<CL_Vector*> ret_list;
-  //CL_Link* a_link = NULL;
 
   for (int map_x = 0; map_x < MAP_WIDTH; map_x++) {
     for (int map_y = 0; map_y < MAP_HEIGHT; map_y++) {
       if (ratings[map_x][map_y] > 0) {
-	std::list<CL_Vector*>::iterator a_link = ret_list.end();
-        for (std::list<CL_Vector*>::iterator list_iter = ret_list.begin();
-             list_iter != ret_list.end();
-             list_iter++) {
-	  //if ((int)ret_list.get_item(i+1)->z <= ratings[map_x][map_y]) {
-          if ((int)(*list_iter)->z <= ratings[map_x][map_y]) {
-            //a_link = ret_list.get_link(i);
-            a_link = list_iter;
-            break;
-          }
-          else {
-            //a_link = ret_list.get_link(i+1);
-            a_link = list_iter;
-            a_link++;
+        std::list<CL_Vector*>::iterator a_link = ret_list.end();
+        if (ret_list.size() > 0) {
+          for (std::list<CL_Vector*>::iterator list_iter = ret_list.begin();
+               list_iter != ret_list.end();
+               list_iter++) {
+            if ((int)(*list_iter)->z <= ratings[map_x][map_y]) {
+              a_link = list_iter;
+              break;
+            }
           }
 	}
 	ret_list.insert(a_link, new CL_Vector(map_x, map_y, ratings[map_x][map_y]));
-	//a_link = NULL;
       }
     }
   }
@@ -964,7 +961,8 @@ std::vector<CL_Vector*>* Controller_AI_mass::seek_util()
     }*/
 
   for (std::list<CL_Vector*>::iterator iter = ret_list.begin();
-       iter != ret_list.begin(); iter++) {
+       iter != ret_list.end(); iter++) {
+    assert(*iter != NULL);
     ret_path = path(b_v, *(*iter), RATING_SURE, RATING_SURE);
 
     if (ret_path != NULL) {
@@ -973,9 +971,8 @@ std::vector<CL_Vector*>* Controller_AI_mass::seek_util()
   }
 
   for (std::list<CL_Vector*>::iterator iter = ret_list.begin();
-       iter != ret_list.begin(); iter++) {
+       iter != ret_list.end(); iter++) {
     delete *iter;
-    iter = ret_list.erase(iter);
   }
 
   return ret_path;
@@ -1665,7 +1662,6 @@ static std::vector<CL_Vector*>* reversel(std::vector<CL_Vector*>* list)
   }
 
   std::vector<CL_Vector*>* rev_list = new std::vector<CL_Vector*>;
-  CL_Vector* vec = NULL;
 
   for (std::vector<CL_Vector*>::reverse_iterator iter = list->rbegin();
        iter != list->rend(); iter++) {
